@@ -5,6 +5,8 @@ use vars qw(@ISA $VERSION @EXPORT );
 use Net::DNS;
 use Carp;
 
+use bytes;
+
 use Crypt::OpenSSL::DSA;
 use Crypt::OpenSSL::RSA;
 use Crypt::OpenSSL::Bignum;
@@ -13,12 +15,11 @@ use File::Basename;
 use MIME::Base64;
 use Math::BigInt;
 use Time::Local;
-use Digest::SHA1 qw (sha1);
 
 
 require Exporter;
 
-$VERSION = do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
+$VERSION = do { my @r=(q$Revision: 527 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
 
 sub new {
     my ($class,  $key_file) = @_;
@@ -102,6 +103,20 @@ sub new {
 	      $Exponent2,
 	      $Coefficient,
 	);
+
+      # Trying to determine the keytag
+
+
+      my $keytag_from_data1=$self->dump_rsa_keytag(256,1);
+      my $keytag_from_data2=$self->dump_rsa_keytag(257,1);
+      if (($self->{"keytag"} != $keytag_from_data1) &&
+	  ($self->{"keytag"} != $keytag_from_data2)){
+	  warn "NB: filename seems to have the wrong keytag.\n".
+	      "Depending on DNSKEY RR flags set for this key the keytag should be\n".
+	      $keytag_from_data1. " or ".  $keytag_from_data2. " instead of ".$self->{"keytag"}."\n";
+	  return(0);
+      }
+      
     }elsif ($self->{"algorithm"} == 3){  #DSA
 	my $private_dsa = Crypt::OpenSSL::DSA->new();
 	$private_dsa->set_p($prime_p);
@@ -111,7 +126,6 @@ sub new {
 	$private_dsa->set_pub_key($public_val_y);
 	$self->{"privatekey"}=$private_dsa;
     }
-    
     return $self;
 
 }
@@ -237,7 +251,6 @@ sub  dump_rsa_pub {
 
 sub dump_rsa_keytag{
     my $self=shift;
-    
     my $flags;
     if (defined $self->{"flags"}){
 	$flags=$self->{"flags"}
@@ -245,7 +258,12 @@ sub dump_rsa_keytag{
 	$flags=shift;
     }
     return()  unless defined $flags;
-    $self->{"flags"}=$flags; # So this will set flag if empty before
+
+    # This will set flag if empty before, note the undocumented
+    # feature that a non-zero second argument to this function will
+    # _not_ set the flag.
+    $self->{"flags"}=$flags unless shift; 
+
     my $key=$self->dump_rsa_pub;
     return ()  unless $key;
     my $tmprr=Net::DNS::RR->new("tmp  IN DNSKEY $flags 3 5 $key");
@@ -301,7 +319,7 @@ Net::DNS::SEC::Private - DNS SIG Private key object
 
 =head1 SYNOPSIS
 
-C<use Net::DNS::SEC::Private>;
+use Net::DNS::SEC::Private;
 my $private=Net::DNS::SEC::Private->new($keypath);
 
 =head1 DESCRIPTION
@@ -392,7 +410,6 @@ Returns an empty string on failure.
 This function will calculate the keytag assuming that key is a used
 with the RSASHA1 signing algorithm and with the DNSKEY flags as input.
 
-
 The flags field may be needed in case it was not specified when the
 key was created. If the object allready knows it's flags vallue the
 input is ignored. 
@@ -458,7 +475,7 @@ Then a self signature over the public key is created and verified.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2002, 2003  RIPE NCC.  Author Olaf M. Kolkman <net-dns-sec@ripe.net>
+Copyright (c) 2002-2005 RIPE NCC.  Author Olaf M. Kolkman <olaf@net-dns.org>
 
 All Rights Reserved
 
@@ -484,8 +501,9 @@ This code uses Crypt::OpenSSL which uses the openssl library
 
 =head1 SEE ALSO
 
-L<perl(1)>, L<Net::DNS>, L<Net::DNS::RR::SIG>, L<Crypt::OpenSSL::RSA>,L<Crypt::OpenSSL::DSA>
-RFC 2435 Section 4, RFC 2931.
+L<http://www.net-dns.org/>
+
+L<perl(1)>, L<Net::DNS>, L<Net::DNS::RR::SIG>, L<Crypt::OpenSSL::RSA>,L<Crypt::OpenSSL::DSA>, RFC 2435 Section 4, RFC 2931.
 
 =cut
 
